/**
 * Created by Grigorii Shovkoplias on 11.03.2016.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        genManual();

        for (int i = 0; i < 3; i++) {
            genRand(rand(100) + 1, 100);
            genRandFront(rand(100) + 1, 100);
            genRandBack(rand(100) + 1, 100);
            genSame(rand(100) + 1, rand(100) + 1);
        }

        for (int i = 0; i < 3; i++) {
            genRand(rand(MAXN1) + 1, MAXX);
            genRandFront(rand(MAXN1) + 1, MAXX);
            genRandBack(rand(MAXN1) + 1, MAXX);
            genSame(rand(MAXN1) + 1, rand(MAXX) + 1);
        }

        genRand(MAXN1, MAXX);
        genRandFront(MAXN1, MAXX);
        genRandBack(MAXN1, MAXX);
        genSame(MAXN1, MAXX);

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d and m &lt;= %d and no deletions\"", 1, MAXN1, MAXX));
        desc.println(String.format("bonus_subtask%d=%d", 1, 47));

        //-------------------------second group-------------------------------------------------

        subtask = "subtask2";

        for (int i = 0; i < 3; i++) {
            genRand(rand(MAXN2) + 1, 100);
            genRandFront(rand(MAXN2) + 1, 100);
            genRandBack(rand(MAXN2) + 1, 100);
            genSame(rand(MAXN2) + 1, rand(100) + 1);
        }

        for (int i = 0; i < 3; i++) {
            genRand(rand(MAXN2) + 1, MAXX);
            genRandFront(rand(MAXN2) + 1, MAXX);
            genRandBack(rand(MAXN2) + 1, MAXX);
            genSame(rand(MAXN2) + 1, rand(MAXX) + 1);
        }

        genRand(MAXN2, MAXX);
        genRandFront(MAXN2, MAXX);
        genRandBack(MAXN2, MAXX);
        genSame(MAXN2, MAXX);


        desc.println(String.format("comment_subtask%d=\"n &lt;= %d and m &lt;= %d\"", 2, MAXN2, MAXX));
        desc.println(String.format("bonus_subtask%d=%d", 2, 53));


        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN1 = 1000;
    private final int MAXN2 = 100_000;
    private final int MAXX = 1_000_000_000;
    private final int SQRT_MAXX = 10_000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private boolean coin() {
        return rnd.nextBoolean();
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }


    private void print(int test[]) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(test.length);
        for (int i = 0; i < test.length - 1; i++) {
            out.print(test[i] + " ");
        }
        out.println(test[test.length - 1]);
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("3");
        out.println("6 12 8");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("5");
        out.println("2 2 3 4 6");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
    }


    private void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rand(a.length);
            int tmp = a[j];
            a[j] = a[i];
            a[i] = tmp;
        }
    }

    //begin generators

    private void genManual() throws IOException {
        print(new int[]{6, 24, 12, 8});
        print(new int[]{1, 1});
        print(new int[]{345, 213});
        print(new int[]{1234, 5678, 91011});
        print(new int[]{4, 4, 1});
    }

    private void genRand(int n, int max) throws IOException {
        int[] test = new int[n];
        int divider = rand((int)Math.sqrt(max)) + 1;
        for (int i = 0; i < n; i++)
            test[i] = (rand((int)Math.sqrt(max)) + 1) * divider;
        test[0] = rand(max) + 1;
        shuffle(test);
        print(test);
        printDescT(String.format("Random: n = %d, gcd = %d", n, divider));
    }

    private void genRandFront(int n, int max) throws IOException {
        int[] test = new int[n];
        int divider = rand((int)Math.sqrt(max)) + 1;
        for (int i = 0; i < n; i++)
            test[i] = (rand((int)Math.sqrt(max)) + 1) * divider;
        test[0] = rand(max) + 1;
        print(test);
        printDescT(String.format("Random front : n = %d, gcd = %d", n, divider));
    }

    private void genRandBack(int n, int max) throws IOException {
        int[] test = new int[n];
        int divider = rand((int)Math.sqrt(max)) + 1;
        for (int i = 0; i < n; i++)
            test[i] = (rand((int)Math.sqrt(max)) + 1)* divider;
        test[n - 1] = rand(max) + 1;
        print(test);
        printDescT(String.format("Random back: n = %d, gcd = %d", n, divider));
    }

    private void genSame(int n, int x) throws IOException {
        int[] test = new int[n];
        for (int i = 0; i < n; i++)
            test[i] = x;
        print(test);
        printDescT(String.format("Same: n = %d, a_i = %d", n, x));
    }


    //end generators

}
